/**
 * Copyright (c) 2021, WSO2 Inc. (http://www.wso2.com). All Rights Reserved.
 *
 * This software is the property of WSO2 Inc. and its suppliers, if any.
 * Dissemination of any information or reproduction of any material contained
 * herein in any form is strictly forbidden, unless permitted by WSO2 expressly.
 * You may not alter or remove any copyright or other notice from copies of this content."
 */

import { IdentityAppsApiException } from "@wso2is/core/exceptions";
import { AlertInterface, AlertLevels, IdentifiableComponentInterface } from "@wso2is/core/models";
import { addAlert } from "@wso2is/core/store";
import {
    DocumentationLink,
    GenericIcon,
    Hint,
    PageLayout,
    useDocumentation,
    useWindowDimensions
} from "@wso2is/react-components";
import cloneDeep from "lodash-es/cloneDeep";
import isEmpty from "lodash-es/isEmpty";
import merge from "lodash-es/merge";
import React, { Fragment, FunctionComponent, ReactElement, SyntheticEvent, useEffect, useState } from "react";
import { useTranslation } from "react-i18next";
import { useDispatch, useSelector } from "react-redux";
import { Checkbox, CheckboxProps, Divider, Responsive } from "semantic-ui-react";
import { AppState } from "../../../../features/core/store";
import { getBrandingPreference, updateBrandingPreference } from "../api";
import { BrandingPreferenceTabs, DesignFormValuesInterface } from "../components";
import { BrandingPreferencesConstants } from "../constants";
import { BrandingPreferenceMeta } from "../meta";
import { BrandingPreferenceAPIResponseInterface, BrandingPreferenceInterface } from "../models";

/**
 * Proptypes for the branding page component.
 */
type BrandingPageInterface = IdentifiableComponentInterface;

/**
 * Branding page.
 *
 * @param {BrandingPageInterface} props - Props injected to the component.
 *
 * @return {React.ReactElement}
 */
const BrandingPage: FunctionComponent<BrandingPageInterface> = (
    props: BrandingPageInterface
): ReactElement => {

    const {
        ["data-componentid"]: componentId
    } = props;

    const { t } = useTranslation();

    const dispatch = useDispatch();

    const { getLink } = useDocumentation();
    
    const { width } = useWindowDimensions();

    const tenantDomain: string = useSelector((state: AppState) => state.auth.tenantDomain);

    const [ isBrandingConfigured, setIsBrandingConfigured ] = useState<boolean>(true);
    const [
        brandingPreference,
        setBrandingPreference
    ] = useState<BrandingPreferenceInterface>(BrandingPreferencesConstants.DEFAULT_PREFERENCE);
    const [
        isBrandingPreferenceFetchRequestLoading,
        setIsBrandingPreferenceFetchRequestLoading
    ] = useState<boolean>(undefined);
    const [
        isBrandingPreferenceUpdateRequestLoading,
        setIsBrandingPreferenceUpdateRequestLoading
    ] = useState<boolean>(undefined);
    const [ isTabletViewport, setIsTabletViewport ] = useState<boolean>(false);

    useEffect(() => {

        if (width < Responsive.onlyTablet.maxWidth) {
            setIsTabletViewport(true);
        }
    }, [ width ]);

    /**
     * Get Branding Preference when the default tenant is resolved.
     */
    useEffect(() => {

        if (!tenantDomain) {
            return;
        }

        setIsBrandingPreferenceFetchRequestLoading(true);

        getBrandingPreference(tenantDomain)
            .then((response: BrandingPreferenceAPIResponseInterface) => {
                if (response instanceof IdentityAppsApiException) {
                    dispatch(addAlert<AlertInterface>({
                        description: t("extensions:develop.branding.notifications.fetch.invalidStatus.description",
                            { tenant: tenantDomain }),
                        level: AlertLevels.ERROR,
                        message: t("extensions:develop.branding.notifications.fetch.invalidStatus.message")
                    }));

                    return;
                }

                // Check if the returned Branding preference is of the intended tenant.
                if (response.name !== tenantDomain) {
                    dispatch(addAlert<AlertInterface>({
                        description: t("extensions:develop.branding.notifications.fetch.tenantMismatch.description",
                            { tenant: tenantDomain }),
                        level: AlertLevels.ERROR,
                        message: t("extensions:develop.branding.notifications.fetch.tenantMismatch.message")
                    }));

                    return;
                }

                setIsBrandingConfigured(true);
                setBrandingPreference(response.preference);
            })
            .catch((error: IdentityAppsApiException) => {
                // Check if Branding is not configured for the tenant. If so, silent the errors.
                if (error.code === BrandingPreferencesConstants.BRANDING_NOT_CONFIGURED_ERROR_CODE) {
                    setIsBrandingConfigured(false);
                    setBrandingPreference(BrandingPreferencesConstants.DEFAULT_PREFERENCE);

                    return;
                }

                dispatch(addAlert<AlertInterface>({
                    description: t("extensions:develop.branding.notifications.fetch.genericError.description",
                        { tenant: tenantDomain }),
                    level: AlertLevels.ERROR,
                    message: t("extensions:develop.branding.notifications.fetch.genericError.message")
                }));

                setBrandingPreference(BrandingPreferencesConstants.DEFAULT_PREFERENCE);
            })
            .finally(() => {
                setIsBrandingPreferenceFetchRequestLoading(false);
            });
    }, [ tenantDomain ]);

    /**
     * Handles preference form submit action.
     * @param {MultiFactorAuthenticatorInterface} values - Form values.
     */
    const handlePreferenceFormSubmit = <T extends any>(values: T): void => {

        _updateBrandingPreference(merge(cloneDeep(brandingPreference), values), isBrandingConfigured);
    };

    /**
     * Function to update the Branding Preferences using the API.
     *
     * @param {BrandingPreferenceInterface} preference - Preference object.
     * @param {boolean} _isBrandingConfigured - Local flag passed as a param to determine if branding is configured.
     */
    const _updateBrandingPreference = (preference: BrandingPreferenceInterface,
        _isBrandingConfigured: boolean): void => {

        setIsBrandingPreferenceUpdateRequestLoading(true);

        updateBrandingPreference(_isBrandingConfigured, tenantDomain, preference)
            .then((response: BrandingPreferenceAPIResponseInterface) => {
                if (response instanceof IdentityAppsApiException) {
                    dispatch(addAlert<AlertInterface>({
                        description: t("extensions:develop.branding.notifications.update.invalidStatus.description",
                            { tenant: tenantDomain }),
                        level: AlertLevels.ERROR,
                        message: t("extensions:develop.branding.notifications.update.invalidStatus.message")
                    }));

                    return;
                }

                // Check if the returned Branding preference is of the intended tenant.
                if (response.name !== tenantDomain) {
                    dispatch(addAlert<AlertInterface>({
                        description: t("extensions:develop.branding.notifications.update.tenantMismatch.description",
                            { tenant: tenantDomain }),
                        level: AlertLevels.ERROR,
                        message: t("extensions:develop.branding.notifications.update.tenantMismatch.message")
                    }));

                    return;
                }

                setIsBrandingConfigured(true);
                setBrandingPreference(response.preference);

                dispatch(addAlert<AlertInterface>({
                    description: t("extensions:develop.branding.notifications.update.success.description",
                        { tenant: tenantDomain }),
                    level: AlertLevels.SUCCESS,
                    message: t("extensions:develop.branding.notifications.update.success.message")
                }));
            })
            .catch((error: IdentityAppsApiException) => {
                // Edge Case...Try again with POST, if Branding preference has been removed due to concurrent sessions.
                if (error.code === BrandingPreferencesConstants.BRANDING_NOT_CONFIGURED_ERROR_CODE) {
                    setIsBrandingConfigured(false);
                    _updateBrandingPreference(preference, false);

                    return;
                }

                dispatch(addAlert<AlertInterface>({
                    description: t("extensions:develop.branding.notifications.update.genericError.description",
                        { tenant: tenantDomain }),
                    level: AlertLevels.ERROR,
                    message: t("extensions:develop.branding.notifications.update.genericError.message")
                }));

                setBrandingPreference(BrandingPreferencesConstants.DEFAULT_PREFERENCE);
            })
            .finally(() => {
                setIsBrandingPreferenceUpdateRequestLoading(false);
            });
    };

    /**
     * Handles the publish/un-publish toggle onchange callback.
     * @param {SyntheticEvent} e - Event.
     * @param {CheckboxProps} data - Event Data.
     */
    const handlePublishToggle = (e: SyntheticEvent, data: CheckboxProps): void => {

        _updateBrandingPreference(merge(cloneDeep(brandingPreference), {
            configs: {
                isBrandingEnabled: data.checked
            }
        }), isBrandingConfigured);
    };

    /**
     * Renders the publish toggle.
     * @return {ReactElement}
     */
    const renderPublishToggle = (): ReactElement => {

        return (
            <Fragment>
                <Checkbox
                    toggle
                    label={ t("extensions:develop.branding.publishToggle.label") }
                    checked={ brandingPreference.configs?.isBrandingEnabled }
                    onChange={ handlePublishToggle }
                />
                <Hint popup>{ t("extensions:develop.branding.publishToggle.hint") }</Hint>
            </Fragment>
        );
    };

    return (
        <PageLayout
            action={ !isTabletViewport && renderPublishToggle() }
            image={ (
                <GenericIcon
                    square
                    relaxed
                    transparent
                    bordered
                    rounded
                    icon={
                        !isEmpty(brandingPreference.images?.logo?.imgURL)
                            ? brandingPreference.images.logo.imgURL
                            : BrandingPreferenceMeta.getBrandingPreferenceFallbacks().images.logo.imgURL
                    }
                    size="tiny"
                />
            ) }
            title={ tenantDomain }
            isLoading={
                !tenantDomain
                || (
                    isBrandingPreferenceFetchRequestLoading === undefined
                    || isBrandingPreferenceFetchRequestLoading === true
                )
            }
            description={ (
                <div className="with-label">
                    { t("extensions:develop.branding.pageHeader.description") }
                    <DocumentationLink
                        link={ getLink("develop.applications.learnMore") }
                    >
                        { t("common:learnMore") }
                    </DocumentationLink>
                </div>
            ) }
            data-testid={ `${ componentId }-layout` }
            className="branding-page"
        >
            {
                isTabletViewport && (
                    <Fragment>
                        { renderPublishToggle() }
                        <Divider hidden />
                    </Fragment>
                )
            }
            <BrandingPreferenceTabs
                brandingPreference={ brandingPreference }
                isLoading={
                    !tenantDomain
                    || (
                        isBrandingPreferenceFetchRequestLoading === undefined
                        || isBrandingPreferenceFetchRequestLoading === true
                    )
                }
                isUpdating={ isBrandingPreferenceUpdateRequestLoading }
                onSubmit={ (values: DesignFormValuesInterface) => {
                    handlePreferenceFormSubmit<DesignFormValuesInterface>(values);
                } }
                isSplitView={ !isTabletViewport }
            />
        </PageLayout>
    );
};

/**
 * Default props for the component.
 */
BrandingPage.defaultProps = {
    "data-componentid": "branding-page"
};

/**
 * A default export was added to support React.lazy.
 * TODO: Change this to a named export once react starts supporting named exports for code splitting.
 * @see {@link https://reactjs.org/docs/code-splitting.html#reactlazy}
 */
export default BrandingPage;
